package model;

import app.EvolutionAPI;

import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Member of Population
 */
public class Individual {

    private String chromosome;

    private Double quality;

    /**
     * Generate random chromosome like String. (Don't use loop over all gens of chromosome)
     * @param length - length of chromosome
     * @param countOfPositiveGens - count of "1" in chromosome. Other items are "0"
     */
    public void generateChromosome(int length, int countOfPositiveGens) {
        StringBuilder chromosome = new StringBuilder(length);

        Set<Integer> generatedPositions = getRandomPositions(length, countOfPositiveGens);

        for (int i = 0; i < length; i++) {
            if (generatedPositions.contains(Integer.valueOf(i))) {
                chromosome.append(1);
            } else {
                chromosome.append(0);
            }
        }
        this.chromosome = chromosome.toString();
    }

    /**
     * Helper method which return Set of positions from interval 0 - size of chromosome.
     * @param length - length of String
     * @param count - size of result Set
     * @return - Set with positions
     */
    public Set<Integer> getRandomPositions(int length, int count) {
        Set<Integer> generated = new LinkedHashSet<>(count);
        while (generated.size() < count) {
            Integer next = EvolutionAPI.randomGenerator.nextInt(length);
            if (generated.contains(next)) {
                next = EvolutionAPI.randomGenerator.nextInt(length);
            }
            generated.add(next);
        }
        return generated;
    }

    public Double getQuality() {
        return quality;
    }

    public void setQuality(Double quality) {
        this.quality = quality;
    }

    public String getChromosome() {
        return chromosome;
    }

    public void setChromosome(String chromosome) {
        this.chromosome = chromosome;
    }
}
