package model;

import app.EvolutionAPI;

import java.util.ArrayList;
import java.util.List;

/**
 * Population
 */
public class Population {

    private Double crossRate;
    private Double mutationRate;
    private Integer size;

    private Integer lengthOfChromosome;
    private Integer countOfPositiveGens;

    private Integer socialPressure;
    private boolean penalty;

    private List<Individual> individuals = new ArrayList<>();
    private Individual bestIndividual;
    private Individual worstIndividual;

    public Population(Population oldPopulation) {
        this.crossRate = oldPopulation.getCrossRate();
        this.mutationRate = oldPopulation.getMutationRate();
        this.size = oldPopulation.getSize();

        this.lengthOfChromosome = oldPopulation.getLengthOfChromosome();

        this.countOfPositiveGens = oldPopulation.getCountOfPositiveGens();
        this.socialPressure = oldPopulation.socialPressure;
        this.penalty = oldPopulation.penalty;
    }

    public Population(SetupEvolutionDTO setupProcess) {
        this.crossRate = setupProcess.getCrossRate();
        this.mutationRate = setupProcess.getMutationRate();
        this.size = setupProcess.getSizeOfPopulation();

        this.lengthOfChromosome = setupProcess.getLengthOfChromosome();

        this.countOfPositiveGens = setupProcess.getCountOfPositiveGens();
        this.socialPressure = setupProcess.getSocialPressure();
        this.individuals = new ArrayList<>(size);
        this.penalty = setupProcess.isPenalty();
    }

    public Double getCrossRate() {
        return crossRate;
    }

    public Double getMutationRate() {
        return mutationRate;
    }

    public Integer getSize() {
        return size;
    }

    public List<Individual> getIndividuals() {
        return individuals;
    }

    public Integer getLengthOfChromosome() {
        return lengthOfChromosome;
    }

    public Integer getCountOfPositiveGens() {
        return countOfPositiveGens;
    }

    public Integer getSocialPressure() {
        return socialPressure;
    }

    public Individual getBestIndividual() {
        return bestIndividual;
    }

    public void setBestIndividual(Individual bestIndividual) {
        this.bestIndividual = bestIndividual;
    }

    public Individual getWorstIndividual() {
        return worstIndividual;
    }

    public void setWorstIndividual(Individual worstIndividual) {
        this.worstIndividual = worstIndividual;
    }

    public boolean isPenalty() {
        return penalty;
    }

    /**
     * Create all individual for first usage
     */
    public void initPopulation() {

        for (int i = 0; i < size; i++) {
            individuals.add(createRandomIndividual());
        }
    }

    public Individual createRandomIndividual() {
        Individual individual = new Individual();
        //generate random chromosome
        int randomInt = EvolutionAPI.randomGenerator.nextInt(countOfPositiveGens -1) + 1;
        individual.generateChromosome(lengthOfChromosome, randomInt);
        //set random quality
        individual.setQuality(100.0*Math.random());
        return individual;
    }
}
