package service.impl.crossover;

import model.Individual;
import service.api.CrossoverService;
import utils.EvolutionUtils;
import app.EvolutionAPI;

/**
 * Pri krizeni se aplikuje nahodne vygenerovana maska. Jako oprace se pouzije XOR
 */
public class BinaryMaskCrossover implements CrossoverService {

    @Override
    public Individual[] crossover(Individual parent1, Individual parent2) {
        boolean[] randomBinaryMask = generateRandomBinaryMask(parent1.getChromosome().length());

        //create offspring
        Individual[] children = new Individual[2];
        children[0] = EvolutionUtils.createIndividual(applyMask(parent1, randomBinaryMask));
        children[1]= EvolutionUtils.createIndividual(applyMask(parent2, randomBinaryMask));
        return children;
    }

    private String applyMask(Individual parent, boolean[] mask) {
        String parentChromosome = parent.getChromosome();

        StringBuilder childChromosome = new StringBuilder();
        for (int i = 0; i < mask.length; i++) {
            boolean gen = parentChromosome.charAt(i) == '1';
            boolean maskGen = mask[i];

            //XOR
            if ((gen && maskGen) || (!gen && !maskGen)) {
                childChromosome.append(0);
            } else {
                childChromosome.append(1);
            }
        }
        return childChromosome.toString();
    }

    private boolean[] generateRandomBinaryMask(int length) {
        boolean[] mask = new boolean[length];
        for (int i = 0; i < length; i++) {
            mask[i] = EvolutionAPI.randomGenerator.nextBoolean();
        }
        return mask;
    }
}
