package service.impl.selection;

import app.EvolutionAPI;
import model.Individual;
import org.apache.commons.lang.math.DoubleRange;
import service.api.SelectionService;
import utils.QualityComparator;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Vytvorit primku s jedinci a lepsi jedinec zabira vic mista a pravdepodobnoust ze si vybereme prave jeho je vyssi
 */
public class ProbabilityLineSelection implements SelectionService {
    @Override
    public Individual randomSelectIndividual(List<Individual> individuals, double sumOfFitnessFunction) {
        //firstly sort by quality
        Collections.sort(individuals,new QualityComparator());
        //generate line
        Map<DoubleRange, Individual> lineMap = generateQualityLine(individuals, sumOfFitnessFunction);
        int selectPoint = EvolutionAPI.randomGenerator.nextInt((int) sumOfFitnessFunction - 1) + 1;
        for (DoubleRange range : lineMap.keySet()) {
            if (range.containsInteger(selectPoint)) {
                return lineMap.get(range);
            }
        }
        return null;
    }

    private Map<DoubleRange, Individual> generateQualityLine(List<Individual> individuals, double sumOfFitnessFunction) {
        Map<DoubleRange, Individual>  result = new HashMap<>();
        double lastRange = sumOfFitnessFunction;
        for (Individual individual : individuals) {
            Double individualQuality = individual.getQuality();
            DoubleRange range;
            //for last the worst individual set end of range as "0"
            if (individualQuality.equals(0d)) {
                range = new DoubleRange(lastRange, 0d);
            } else {
                range = new DoubleRange(lastRange, lastRange - individualQuality);
            }

            lastRange -= individualQuality;
            result.put(range, individual);
        }
        return result;
    }
}
